@@ -44,7 +44,7 @@ void dedupe_list(PMaster master) {
 		/* step 2, sort em */
 		ssort(array, master->nritems, 1);  /* start with depth of 1 cause all start with < */
 
-		TimerFunc(TIMER_TIMEONLY, 0L, master->msgs);
+		/* TimerFunc(TIMER_TIMEONLY, 0L, master->msgs); */
 		
 		
 		/* step 3, mark dupes */
