@@ -103,10 +103,11 @@ enum { PARAM_HILINE, PARAM_LOWLINE, PARAM_NRGRPS, PARAM_GROUP, PARAM_QUOTE, PARA
 
 
 /*--------------------------------------------------------------------------*/
-PKillStruct parse_killfile(const char *filename, int logfile_yn, int debug, int ignore_postfix) {
+PKillStruct parse_killfile(int which, int logfile_yn, int debug, int ignore_postfix) {
 	FILE *fptr;
 	char buf[MAXLINLEN+1];
 	int i, doprg = FALSE, mastergrp = RETVAL_OK, retval = TRUE;
+	const char *filename;
 #ifdef PERL_EMBED
 	int doperl = FALSE;
 #endif
@@ -136,8 +137,9 @@ PKillStruct parse_killfile(const char *filename, int logfile_yn, int debug, int
 		Masterkill->bodylen = 0;
 		Masterkill->use_extended_regex = FALSE;
 		Masterkill->xover_log_long = FALSE;
-		Masterkill->xoverview = NULL; /* these last three are used in xoverview code */
-		
+/* initialize the master struct */
+		Masterkill->child.Stdin = Masterkill->child.Stdout = -1;
+		Masterkill->child.Pid = -1;		
 		initialize_one_kill(&(Masterkill->master));
 
 #ifdef PERL_EMBED
@@ -148,6 +150,8 @@ PKillStruct parse_killfile(const char *filename, int logfile_yn, int debug, int
 		/* which option to we use in call to full_path() ?  Do we use the postfix or not? */
 		Masterkill->ignore_postfix = ( ignore_postfix == TRUE) ? FP_GET_NOPOSTFIX : FP_GET ;
 
+		filename = ( which == KILL_XOVER ) ? N_XOVER : N_KILLFILE;
+
 		if(debug == TRUE) {
 			do_debug("Trying to read killfile: %s\n", full_path(Masterkill->ignore_postfix, FP_DATADIR, filename));
 		}
@@ -177,7 +181,7 @@ PKillStruct parse_killfile(const char *filename, int logfile_yn, int debug, int
 #ifdef PERL_EMBED
 					if(Masterkill->perl_int == NULL) {
 #endif
-						doprg = killprg_forkit(Masterkill, &buf[Params[PARAM_PROGRAM].len], debug);
+						doprg = killprg_forkit(Masterkill, &buf[Params[PARAM_PROGRAM].len], debug, which);
 #ifdef PERL_EMBED
 					}
 #endif
@@ -198,7 +202,7 @@ PKillStruct parse_killfile(const char *filename, int logfile_yn, int debug, int
 #ifdef PERL_EMBED				
 				if(strncmp(buf, Params[PARAM_PERL].name, (size_t) Params[PARAM_PERL].len) == 0) {
 					if(doprg == FALSE) {
-						doperl = killperl_setup(Masterkill, &buf[Params[PARAM_PERL].len], debug);
+						doperl = killperl_setup(Masterkill, &buf[Params[PARAM_PERL].len], debug, which);
 					}
 				}
 #endif	
@@ -337,11 +341,10 @@ int pass_two(PKillStruct killp, int debug, const char *filename ) {
 void free_killfile(PKillStruct master) {
 
 	int i;
-	POverview tmp, tmp2;
-	
+		
 	if(master != NULL) {
 		/* first kill off killprg if its there */
-		if(master->killfunc==chk_msg_kill_fork) {
+		if(master->killfunc==chk_msg_kill_fork || master->child.Pid != -1) {
 			killprg_closeit(master);
 		}
 #ifdef PERL_EMBED
@@ -361,15 +364,7 @@ void free_killfile(PKillStruct master) {
 			}
 			free(master->grps);
 		}
-		tmp = master->xoverview;
-		while (tmp != NULL) {
-			tmp2 = tmp->next;
-			if(tmp->header != NULL) {
-				free(tmp->header);
-			}
-			free(tmp);
-			tmp = tmp2;
-		}
+
 		free(master);
 	}
 }
@@ -431,7 +426,7 @@ int get_one_article_kill(PMaster master, int logcount, long itemon) {
 	
 	retval =get_chunk_mem(master, &len, CHUNK_HEADER, &inbuf);
 	
-	/* the killfunc pointer points to either chk_msg_kill() or chk_msg_kill_fork() */
+	/* the killfunc pointer points to either chk_msg_kill(), chk_msg_kill_fork(), or chk_msg_kill_perl() */
 	/* do we have to download this sucker, is it mandatory? */
 	if(retval == RETVAL_OK && ((master->curr)->mandatory == MANDATORY_YES || (*killp->killfunc)(master, killp, inbuf, len) == FALSE)) {
 		if(master->MultiFile == TRUE) {
@@ -458,7 +453,7 @@ int get_one_article_kill(PMaster master, int logcount, long itemon) {
 					retval = RETVAL_ERROR;
 					error_log(ERRLOG_REPORT, killf_phrases[9], NULL);
 				}
-				if(retval == RETVAL_OK) {
+				if(retval == RETVAL_OK && master->header_only == FALSE) {
 					/* have we already downloaded the body? */
 					if(killp->pbody != NULL) {
 						inbuf = killp->pbody;
@@ -789,7 +784,6 @@ int check_a_group(PMaster master, POneKill killp, char *headerbuf, char **why) {
 			i = 0;
 			tptr = startline;
 			while(i <= killp->maxxref && *tptr != '\n' && *tptr != '\0' ) {
-				/* some news server use space vice comma for separator */
 				if(*tptr == COLON) {
 					i++;
 				}
