@@ -1,6 +1,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
+#include <string.h>
 #include "suck_config.h"
 
 /* create a text file with the default*phrases arrays in them */
@@ -21,6 +22,8 @@ extern const char *default_sucku_phrases[];
 extern const char *default_lmove_phrases[];
 extern const char *default_active_phrases[];
 extern const char *default_batch_phrases[];
+extern const char *default_xover_phrases[];
+extern const char *default_xover_reasons[];
 
 extern int nr_both_phrases;
 extern int nr_rpost_phrases;
@@ -36,6 +39,8 @@ extern int nr_sucku_phrases;
 extern int nr_lmove_phrases;
 extern int nr_active_phrases;
 extern int nr_batch_phrases;
+extern int nr_xover_phrases;
+extern int nr_xover_reasons;
 
 int do_one_array(FILE *, int, const char **);
 int do_phrases_h(void);
@@ -100,6 +105,12 @@ int main(int argc, char *argv[]) {
 		if(retval == 0) {
 			retval = do_one_array(fpo, nr_batch_phrases, default_batch_phrases);
 		}
+		if(retval == 0) {
+			retval = do_one_array(fpo, nr_xover_phrases, default_xover_phrases);
+		}
+		if(retval == 0) {
+			retval = do_one_array(fpo, nr_xover_reasons, default_xover_reasons);
+		}
 		
 		fclose(fpo);
 		if(retval == -1 ) {
@@ -201,6 +212,15 @@ int do_phrases_h(void) {
 		if(i > maxvar) {
 			maxvar = i;
 		}
+		i = count_vars(default_xover_phrases, nr_xover_phrases);
+		if(i > maxvar) {
+			maxvar = i;
+		}
+		i = count_vars(default_xover_reasons, nr_xover_reasons);
+		if(i > maxvar) {
+			maxvar = i;
+		}
+		
 
 		fprintf(fpo, "/* AUTOMATICALLY GENERATED BY makephrases */\n\n");
 		fprintf(fpo, "#define NR_BOTH_PHRASES %d\n", nr_both_phrases);
@@ -217,7 +237,8 @@ int do_phrases_h(void) {
 		fprintf(fpo, "#define NR_LMOVE_PHRASES %d\n", nr_lmove_phrases);
 		fprintf(fpo, "#define NR_ACTIVE_PHRASES %d\n", nr_active_phrases);
 		fprintf(fpo, "#define NR_BATCH_PHRASES %d\n\n", nr_batch_phrases);
-		
+		fprintf(fpo, "#define NR_XOVER_PHRASES %d\n\n", nr_xover_phrases);
+		fprintf(fpo, "#define NR_XOVER_REASONS %d\n\n", nr_xover_reasons);
 
 		fprintf(fpo, "#define PHRASES_MAX_NR_VARS %d\n\n", maxvar);
 
@@ -235,6 +256,8 @@ int do_phrases_h(void) {
 		fprintf(fpo, "extern char *default_lmove_phrases[];\n");
 		fprintf(fpo, "extern char *default_active_phrases[];\n");
 		fprintf(fpo, "extern char *default_batch_phrases[];\n");
+		fprintf(fpo, "extern char *default_xover_phrases[];\n");
+		fprintf(fpo, "extern char *default_xover_reasons[];\n");
 
 		fprintf(fpo, "extern char **both_phrases;\n");
 		fprintf(fpo, "extern char **rpost_phrases;\n");
@@ -250,6 +273,8 @@ int do_phrases_h(void) {
 		fprintf(fpo, "extern char **lmove_phrases;\n");
 		fprintf(fpo, "extern char **active_phrases;\n");
 		fprintf(fpo, "extern char **batch_phrases;\n");
+		fprintf(fpo, "extern char **xover_phrases;\n");
+		fprintf(fpo, "extern char **xover_reasons;\n");
 
 		fclose(fpo);
 
@@ -263,20 +288,21 @@ int do_phrases_h(void) {
 /*--------------------------------------------------------------*/
 int count_vars(const char **phrases, int nr) {
 
-	int i, x, max, nrline, match, nrat;
+	int i, x, max, nrline, match, nrat, len;
 	
 	max = 0;
 	for(i=0;i<nr;i++) {
 		x = 0;
 		nrline = 0;
-		while( phrases[i][x] != '\0' ) {
+		len = strlen(phrases[i]);
+		while( x < len-3 ) {
 			match = 0;
 			if(phrases[i][x] == PHRASES_SEPARATOR && phrases[i][x+1] == PHRASES_VAR_CHAR) {
 				nrat = x+2;
 				while(isdigit(phrases[i][nrat])) {
 					nrat++;
 				}
-				if(nrat > x+2 && phrases[i][nrat] == PHRASES_SEPARATOR) {
+				if((x < len-3) && (nrat > x+2) && (phrases[i][nrat] == PHRASES_SEPARATOR)) {
 					match = 1;
 					nrline++;
 					x = nrat+1;
