@@ -913,7 +913,7 @@ int do_perl(int sockfd, Pargs myargs) {
 	/* this routine is similar to do_batch */
 	int i, x, nrdone, nrok, retval;
 	FILE *fpi_batch, *fpi_msg;
-	char buf[MAXLINLEN+1], file[MAXLINLEN+1];
+	char buf[MAXLINLEN+1], file[MAXLINLEN+1], infilen[MAXLINLEN+1];
 	char *infile, *args[2] = { NULL, NULL};
 	dSP; /* perl stack pointer */
 	SV *fname; /* used to get fname off of perl stack */
@@ -994,6 +994,10 @@ int do_perl(int sockfd, Pargs myargs) {
 				if(myargs->debug == TRUE) {
 					do_debug("got string =%s\n", (infile == NULL) ? "NULL" : infile);
 				}
+				/* have to do the following cause as soon as we do FREETMPS, the */
+				/* data that infile points to will disappear */
+				strcpy(infilen, infile);
+				infile = infilen;
 			}
 			PUTBACK;
 			FREETMPS;
