@@ -32,6 +32,7 @@
 #include <sys/stat.h>
 #include <math.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include "suck_config.h"
 #include "both.h"
@@ -67,6 +68,7 @@ void free_phrases(void);
 int parse_args(PMaster, int, char *[]);
 int get_group_number(PMaster, char *);
 int do_supplemental(PMaster);
+int do_sup_bynr(PMaster, char *);
 #ifdef MYSIGNAL
 RETSIGTYPE sighandler(int);
 void pause_signal(int, PMaster);
@@ -107,7 +109,7 @@ enum {
 	ARG_USERID, ARG_VERSION, ARG_WAIT, ARG_LOCALHOST, ARG_TIMEOUT, ARG_NRMODE, ARG_AUTOAUTH, \
 	ARG_NODEDUPE, ARG_NO_CHK_MSGID, ARG_READACTIVE, ARG_PREBATCH, ARG_SKIP_ON_RESTART, \
 	ARG_KLOG_NAME, ARG_USEGUI, ARG_XOVER, ARG_CONN_DEDUPE, ARG_POST_FILTER, ARG_CONN_ACTIVE,
-	ARG_HIST_FILE,
+	ARG_HIST_FILE, ARG_HEADER_ONLY, ARG_ACTIVE_LASTREAD, ARG_USEXOVER,
 }; 
 
 typedef struct Arglist{
@@ -130,9 +132,11 @@ const Args arglist[] = {
 	{"dd", "dir_data",    1, ARG_DIR_DATA, 37},
 	{"dm", "dir_msgs",    1, ARG_DIR_MSGS, 37},
 	{"e",  "def_error_log", 0, ARG_DEF_ERRLOG, -1},
-	{"f",  "reconnect_dedupe", 0, ARG_CONN_DEDUPE, -1},	
+	{"f",  "reconnect_dedupe", 0, ARG_CONN_DEDUPE, -1},
+	{"g",  "header_only", 0, ARG_HEADER_ONLY, -1},	
 	{"h",  "host", 1, ARG_HOST, 51},
 	{"hl", "localhost", 1, ARG_LOCALHOST, 50},
+	{"i",  "default_activeread", 1, ARG_ACTIVE_LASTREAD, 65},	
 	{"k",  "kill_no_postfix", 0, ARG_NO_POSTFIX, -1},
 	{"l",  "language_file", 1, ARG_LANGUAGE, 47},
 	{"m",  "multifile", 0, ARG_MULTIFILE, -1},
@@ -174,6 +178,8 @@ const Args arglist[] = {
 	{"V",  "version", 0, ARG_VERSION, -1},
 	{"W",  "wait", 2, ARG_WAIT, 46},
 	{"X",  "no_xover", 0, ARG_XOVER, -1},
+	{"Z",  "use_xover", 0, ARG_USEXOVER, -1},
+	
 };
 
 #define MAX_ARG_PARAMS 2	/* max nr of params with any arg */
@@ -186,6 +192,7 @@ int main(int argc, char *argv[]) {
 	Master master;
 	PList temp;
 	PGroups ptemp;
+	POverview pov;
 	char *inbuf, **args, **fargs = NULL;
 	int nr, resp, loop, fargc, retval = RETVAL_OK;	
 
@@ -245,7 +252,11 @@ int main(int argc, char *argv[]) {
 	master.post_filter= NULL;
 	master.conn_active = FALSE;
 	master.history_file = HISTORY_FILE;
+	master.header_only = FALSE;
 	master.db = -1;
+	master.active_lastread = ACTIVE_DEFAULT_LASTREAD;
+	master.use_xover = FALSE;
+	master.xoverview = NULL;
 	
 	/* have to do this next so if set on cmd line, overrides this */
 
@@ -341,6 +352,9 @@ int main(int argc, char *argv[]) {
 		do_debug("master.post_filter = %s\n", null_str(master.post_filter));
 		do_debug("master.conn_active = %s\n", true_str(master.conn_active));
 		do_debug("master.history_file = %s\n", null_str(master.history_file));
+		do_debug("master.header_only = %s\n", true_str(master.header_only));
+		do_debug("master.active_lastread = %d\n", master.active_lastread);
+		do_debug("master.use_xover = %s\n", true_str(master.use_xover));
 		
 #ifdef TIMEOUT
 		do_debug("TimeOut = %d\n", TimeOut);
@@ -417,12 +431,12 @@ int main(int argc, char *argv[]) {
 		/* now parse the killfile */
 #ifdef KILLFILE
 		if(master.do_killfile == TRUE) {
-			master.killp = parse_killfile(N_KILLFILE, master.killfile_log, master.debug, master.kill_ignore_postfix);
+			master.killp = parse_killfile(KILL_KILLFILE, master.killfile_log, master.debug, master.kill_ignore_postfix);
 		}
 #endif
 		/* now parse the xover killfile */
 		if(master.do_xover == TRUE) {
-			master.xoverp = parse_killfile(N_XOVER, master.killfile_log, master.debug, master.kill_ignore_postfix);
+			master.xoverp = parse_killfile(KILL_XOVER, master.killfile_log, master.debug, master.kill_ignore_postfix);
 		}
 		
 		print_phrases(master.msgs, suck_phrases[1], master.host, NULL);		
@@ -431,8 +445,9 @@ int main(int argc, char *argv[]) {
 		}
 		else {	 
 			/* if we have XOVER killfiles, we need to get the format of em before any processing */
-			if(master.xoverp != NULL) {
+			if(master.xoverp != NULL || master.use_xover == TRUE) {
 				get_xoverview(&master);
+				killprg_sendoverview(&master);
 			}
 			
 			/* first, check for restart articles, */
@@ -508,7 +523,7 @@ int main(int argc, char *argv[]) {
 		if(master.debug == TRUE) {
 			do_debug("retval=%d (RETVAL_OK=%d), m.nrgot=%d, m.batch=%d\n", retval, RETVAL_OK, master.nrgot, master.batch);
 		}
-		if((retval == RETVAL_OK || master.always_batch == TRUE) && master.nrgot > 0) {
+		if((retval == RETVAL_OK || master.always_batch == TRUE) && master.nrgot > 0 && master.header_only == FALSE) {
 			switch(master.batch) {
 			  case BATCH_INNXMIT:
 				  do_post_filter(&master);
@@ -554,10 +569,21 @@ int main(int argc, char *argv[]) {
 			free(master.groups);
 			master.groups = ptemp;
 		}
+		/* free up overview.fmt list */
+		while (master.xoverview != NULL) {
+			pov = (master.xoverview)->next;
+			if((master.xoverview)->header != NULL) {
+				free((master.xoverview)->header);
+			}
+			free(master.xoverview);
+			master.xoverview = pov;
+		}
 		
 #ifdef KILLFILE
 		free_killfile(master.killp);
 #endif
+		free_killfile(master.xoverp);
+		
 		if(fargs != NULL) {
 			free_args(fargc, fargs);
 		}		
@@ -807,12 +833,16 @@ int do_one_group(PMaster master, char *buf, char *group, FILE *newrc, long lastr
 				}
 				print_phrases(master->msgs, suck_phrases[15], group, str_long(high-lastread), str_long(lastread+1), str_long(high), NULL);
 				if(master->xoverp != NULL) {
-					/* do this via the xover command */
-					retval = do_group_xover(master, group, lastread+1);
+					/* do this via the xover killfile command */
+					retval = do_group_xover(master, group, lastread+1, high);
 				}
-				if((master->xoverp == NULL) || (retval == RETVAL_ERROR)) {
+				/* do we use xover or xhdr to get our article list */
+				if(master->use_xover == TRUE) {
+					retval = get_xover(master, group, lastread+1, high);
+				}
+				else if((master->xoverp == NULL) || (retval == RETVAL_NOXOVER)) {
 					/* do this the normal way */
-					sprintf(cmd, "xhdr Message-ID %ld-\r\n", lastread+1);
+					sprintf(cmd, "xhdr Message-ID %ld-%ld\r\n", lastread+1,high);
 					i = send_command(master,cmd,&inbuf,221);
 					if(i != RETVAL_OK) {
 						retval = RETVAL_ERROR;
@@ -900,7 +930,7 @@ int get_articles(PMaster master) {
 					
 				        /* to be polite to the server, lets allow for a pause every once in a while */
 					if(master->pause_time > 0 && master->pause_nrmsgs > 0) {
-						if(downloaded % master->pause_nrmsgs == 0) {
+						if((downloaded > 0) && (downloaded % master->pause_nrmsgs == 0)) {
 							sleep(master->pause_time);
 						}
 					}
@@ -944,7 +974,7 @@ int get_articles(PMaster master) {
 
 				/* to be NOT polite to the server reconnect every X msgs to combat the INND */
 				/* LIKE_PULLERS=DONT.  This is last in loop, so if problem can abort gracefully */
-				if(master->reconnect_nr > 0 && downloaded % master->reconnect_nr == 0) {
+				if((master->reconnect_nr > 0) && (downloaded > 0) && (downloaded % master->reconnect_nr == 0)) {
 					retval =  do_connect(master, CONNECT_AGAIN);
 					if(master->curr != NULL) {
 						(master->curr)->sentcmd = FALSE;	/* if we sent command force resend */
@@ -983,21 +1013,77 @@ int do_supplemental(PMaster master) {
 	if((fp = fopen(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL), "r")) != NULL) {
 		print_phrases(master->msgs, suck_phrases[17], NULL);
 		while(retval == RETVAL_OK && fgets(linein, MAXLINLEN, fp) != NULL) {
-			if(linein[0] != '<') {
-				error_log(ERRLOG_REPORT, suck_phrases[18], linein, NULL);
+			if(linein[0] == '!') {
+				retval = do_sup_bynr(master, linein);
+			}
+			else if(linein[0] == '<') {
+				retval = allocnode(master, linein, MANDATORY_YES, NULL, 0L);
 			}
 			else {
-				retval = allocnode(master, linein, MANDATORY_YES, NULL, 0L);
+				error_log(ERRLOG_REPORT, suck_phrases[18], linein, NULL);
 			}
 		}
 		print_phrases(master->msgs, suck_phrases[19], str_int(master->nritems-oldkept), \
 			str_int(master->nritems), NULL);
 		fclose(fp);
 	}
-
-
+	
 	return retval;
 }
+/*------------------------------------------------------------------------------------------*/
+ int do_sup_bynr(PMaster master, char *linein) {
+	 int retval = RETVAL_OK;
+	 /* this routine takes in a line formated !group_name article_nr */
+	 /* gets it's Msg-ID (via XHDR), then adds it to our list of articles */
+	 /* as a mandatory article to download */
+
+	 char grpname[MAX_GRP_LEN], cmd[MAXLINLEN], *resp;
+	 int i, done;
+	 long nrlow, nrhigh;
+	 
+	 if(master->debug == TRUE) {
+		 do_debug("supplemental adding %s", linein);
+	 }
+	 i = sscanf(linein, "!%s %ld-%ld", grpname, &nrlow, &nrhigh);
+	 if(i < 2) {
+		 error_log(ERRLOG_REPORT, suck_phrases[18], linein, NULL);
+	 }
+	 else {
+		 sprintf(cmd,"group %s\r\n",grpname);
+		 if(send_command(master,cmd,NULL,211) == RETVAL_OK) {
+			 if(i == 2) {
+				 sprintf(cmd,"xhdr Message-ID %ld\r\n",nrlow);
+			 }
+			 else {
+				 sprintf(cmd,"xhdr Message-ID %ld-%ld\r\n",nrlow,nrhigh);
+			 }
+			 if(send_command(master,cmd,NULL,221) == RETVAL_OK) {
+				 /* now have to get message-id and the . */
+				 done = FALSE;
+				 while( done == FALSE) {
+					 if(sgetline(master->sockfd, &resp) < 0) {
+						 retval = RETVAL_ERROR;
+						 done = TRUE;
+					 }
+					 else {
+						 if(master->debug == TRUE) {
+							 do_debug("Got %s", resp);
+						 }
+						 if(*resp == '.') {
+							 done = TRUE;
+						 }
+						 else {
+							 retval = allocnode(master, resp, MANDATORY_YES, grpname, 0L);
+						 }
+					 }
+				 }
+			 }
+		 }	 
+	 }
+	 
+	 return retval;
+	 
+ } 
 /*----------------------------------------------------------------------*/
  int get_group_number(PMaster master, char *grpname) {
 
@@ -1224,7 +1310,9 @@ int get_one_article(PMaster master, int logcount, long itemon) {
 	
 	/* first send command to get article if not already sent */
 	if((master->curr)->sentcmd == FALSE) {
-		cmd = build_command(master, "article", master->curr); 
+		cmd = (master->header_only == FALSE)
+			? build_command(master, "article", master->curr)
+			: build_command(master, "head", master->curr); 
 		if(master->debug == TRUE) {
 			do_debug("Sending command: \"%s\"",cmd);
 		}
@@ -1237,7 +1325,9 @@ int get_one_article(PMaster master, int logcount, long itemon) {
 		/* can't use pipelining if nrmode on, cause of GROUP command I'll have to send*/
 			if(plist->sentcmd == FALSE) {
 				/* send next command */
-				cmd = build_command(master, "article", plist);
+				cmd = (master->header_only == FALSE)
+					? build_command(master, "article", plist)
+					: build_command(master, "head", plist); 
 				if(master->debug == TRUE) {
 					do_debug("Sending command: \"%s\"",cmd);
 				}
@@ -1300,7 +1390,8 @@ int get_one_article(PMaster master, int logcount, long itemon) {
 				retval = RETVAL_ERROR;
 			}
 		}	
-		if(nr == 220 ) {
+		/* 221 = header 220 = article */
+		if(nr == 220 || nr == 221 ) {
 			/* get the article */
 			if((retval = get_a_chunk(master->sockfd, fptr)) == RETVAL_OK) {
 				master->nrgot++;
@@ -1321,7 +1412,7 @@ int get_one_article(PMaster master, int logcount, long itemon) {
 	}
 	
 	if(master->MultiFile == TRUE) {
-		if(retval == RETVAL_ERROR || nr != 220) {
+		if(retval == RETVAL_ERROR || (nr != 221 && nr != 220)) {
 			unlink(tname);
 		}
 		/* now rename it to the permanent file name */
@@ -1405,6 +1496,8 @@ int restart_yn(PMaster master) {
 			while(itemon != NULL && done == FALSE) {
 				itemnr++;
 				if(itemon->downloaded == FALSE) {
+					/* so that we don't think we already sent command */
+					itemon->sentcmd = FALSE;
 					if(master->skip_on_restart == TRUE) {
 						/* mark one more */
 						print_phrases(master->msgs, suck_phrases[60], NULL);
@@ -1642,7 +1735,7 @@ int scan_args(PMaster master, int argc, char *argv[]) {
 /*---------------------------------------------------------------------------------------*/
 int parse_args(PMaster master, int arg, char *argv[]) {
 
-	int retval = RETVAL_OK;
+	int x, retval = RETVAL_OK;
 	
 	switch(arg) {
 	case ARG_ALWAYS_BATCH:
@@ -1818,6 +1911,21 @@ int parse_args(PMaster master, int arg, char *argv[]) {
 	case ARG_HIST_FILE:
 		master->history_file = argv[0];
 		break;
+	case ARG_HEADER_ONLY:
+		master->header_only = TRUE;
+		break;
+	case ARG_ACTIVE_LASTREAD:
+		x = atoi(argv[0]);
+		if(x > 0) {
+			error_log(ERRLOG_REPORT, suck_phrases[66], NULL);
+		}
+		else {
+			master->active_lastread = atoi(argv[0]);
+		}
+		break;
+	case ARG_USEXOVER:
+		master->use_xover = TRUE;
+		break;
 #ifdef TIMEOUT
 	case ARG_TIMEOUT:
 		TimeOut = atoi(argv[0]);
@@ -2008,3 +2116,5 @@ void free_phrases(void) {
 		}
 		
 }
+
+		 
