@@ -24,6 +24,15 @@ typedef struct GroupList {
 	struct GroupList *next;
 } Groups, *PGroups;
 
+/* link list for list overview.fmt */
+typedef struct XOVERVIEW {
+	struct XOVERVIEW *next;
+	char *header;  /* dynamically alloced */
+	char *field;
+	int fieldlen;
+	int full;
+} Overview, *POverview;
+
 /* Master Structure */
 typedef struct {
 	PList head;
@@ -60,6 +69,9 @@ typedef struct {
 	int do_xover;
 	int conn_dedupe;
 	int conn_active;
+	int header_only;
+	int active_lastread;
+	int use_xover;
 	unsigned short int portnr;
 	long rnews_size;
 	FILE *msgs;
@@ -81,6 +93,7 @@ typedef struct {
 	int grpnr;
 	void *killp;
 	void *xoverp;
+	POverview xoverview;
 } Master, *PMaster;
 
 int get_a_chunk(int, FILE *);
@@ -94,7 +107,8 @@ const char *true_str(int);
 int allocnode(PMaster, char *, int, char *, long);
 int do_connect(PMaster, int);
 
-enum { RETVAL_ERROR = -1, RETVAL_OK = 0, RETVAL_NOARTICLES, RETVAL_UNEXPECTEDANS, RETVAL_VERNR, RETVAL_NOAUTH, RETVAL_EMPTYKILL };
+enum { RETVAL_ERROR = -1, RETVAL_OK = 0, RETVAL_NOARTICLES, RETVAL_UNEXPECTEDANS, RETVAL_VERNR, \
+       RETVAL_NOAUTH, RETVAL_EMPTYKILL, RETVAL_NOXOVER };
 enum { BATCH_FALSE, BATCH_INNXMIT, BATCH_RNEWS, BATCH_LMOVE, BATCH_INNFEED, BATCH_LIHAVE };		/* poss values for batch variable */
 enum { MANDATORY_YES = 'M' , MANDATORY_OPTIONAL = 'O'};	/* do we have to download this article */
 /* note the MANDATORY_DELETE is used in the dedupe and checkhistory phases to flag em for deletion */
