@@ -23,13 +23,14 @@ typedef struct Grplist {
 int do_one_line(PMaster, char *, char *,PGrpList, int);
 int chk_a_group(PMaster, POneKill, POverview, char **);
 int match_group(char *, char *, int);
+char *find_msgid(PMaster, char *);
 
 /* these MUST match xover_reasons[] ! EXCEPT FOR THE last entry which MUST BE REASON_NONE */
 enum { REASON_HIBYTES, REASON_LOWBYTES, REASON_HILINES, REASON_LOWLINES, REASON_HEADER, REASON_NOKEEP, REASON_TIE,\
-       REASON_XREF, REASON_NONE};
+       REASON_XREF, REASON_PRG, REASON_PERL, REASON_NONE};
 
 /*----------------------------------------------------------------------*/
-int do_group_xover(PMaster master, char *grpname, long startnr) {
+int do_group_xover(PMaster master, char *grpname, long startnr, long endnr) {
 
 	int done, len, nr, retval = RETVAL_OK;
 	char *resp, cmd[MAXLINLEN];
@@ -37,7 +38,7 @@ int do_group_xover(PMaster master, char *grpname, long startnr) {
 	PGroup grpkill = NULL;
 	PGrpList glist = NULL, gat = NULL, gptr;
 	
-	sprintf(cmd, "xover %ld-\r\n", startnr);
+	sprintf(cmd, "xover %ld-%ld\r\n", startnr,endnr);
 	retval = send_command(master, cmd, &resp, 0);
 	if(retval == RETVAL_OK) {
 		number(resp, &nr);
@@ -47,7 +48,7 @@ int do_group_xover(PMaster master, char *grpname, long startnr) {
 		  case 502:
 			  /* abort on these errors */
 			  error_log(ERRLOG_REPORT, xover_phrases[0], resp, NULL);
-			  retval = RETVAL_ERROR;
+			  retval = RETVAL_NOXOVER;
 			  break;
 		  case 420:
 			  /* no articles available, no prob */
@@ -125,7 +126,7 @@ int do_one_line(PMaster master, char *group, char *linein, PGrpList grpskill, in
 	PGroup grpkill;
 	
 	PKillStruct masterk = master->xoverp; /* our master killstruct */
-	overv = masterk->xoverview;
+	overv = master->xoverview;
 	ptr  = get_long(linein, &msgnr);  /* get our message number */
 	
 	/* first go thru and match up the header with the fields in the line, and fill */
@@ -133,6 +134,15 @@ int do_one_line(PMaster master, char *group, char *linein, PGrpList grpskill, in
 	while( *ptr != '\0' && overv != NULL) {
 		/* we are just past a tab */
 		len = 0;
+		if(overv->full == TRUE) {
+			/* have to skip header name in the xoverview */
+			while(*ptr != COLON && *ptr != '\0' && *ptr != '\t') {
+				ptr++;
+			}
+			if(*ptr != '\0') {
+				ptr++; /* move past colon */
+			}
+		}
 		overv->field = ptr;
 		while(*ptr != '\t' && *ptr != '\0') {
 			len++;
@@ -158,49 +168,61 @@ int do_one_line(PMaster master, char *group, char *linein, PGrpList grpskill, in
 	}
 	if(msgid == NULL) {
 		error_log(ERRLOG_REPORT, xover_phrases[13], linein, NULL);
-	}
+	}	
 	else {
-		/* we have to check against master killfile and group kill file */
-		match = chk_a_group( master, &(masterk->master), masterk->xoverview, &reason);
+		if(masterk->child.Pid != -1) {
+		/* send to child program */
+			match = (killprg_sendxover(master, linein) == TRUE) ? REASON_PRG : REASON_NONE;
+		}
+#ifdef PERL_EMBED
+		else if(masterk->perl_int != NULL) {
+			/* send to perl subroutine */			
+			match = (killperl_sendxover(master, linein) == TRUE) ? REASON_PERL : REASON_NONE;
+		}
+#endif
+		else {
+			/* we have to check against master killfile and group kill file */
+			match = chk_a_group( master, &(masterk->master), master->xoverview, &reason);
 		
-		if((grpskill != NULL) && (match == REASON_NONE || masterk->grp_override == TRUE)) {
-			/* we have to check against group */
-			while ( grpskill != NULL) {
-				grpkill = grpskill->grp;
-				match = chk_a_group( master, &(grpkill->match), masterk->xoverview, &reason);
-				if(grpkill->delkeep == DELKEEP_KEEP) {
-					if(match != REASON_NONE) {
-						keep = TRUE;
+			if((grpskill != NULL) && (match == REASON_NONE || masterk->grp_override == TRUE)) {
+				/* we have to check against group */
+				while ( grpskill != NULL) {
+					grpkill = grpskill->grp;
+					match = chk_a_group( master, &(grpkill->match), master->xoverview, &reason);
+					if(grpkill->delkeep == DELKEEP_KEEP) {
+						if(match != REASON_NONE) {
+							keep = TRUE;
+						}
+						else {
+							del = TRUE;
+							why = REASON_NOKEEP;
+						}
 					}
 					else {
-						del = TRUE;
-						why = REASON_NOKEEP;
+						if(match != REASON_NONE) {
+							del = TRUE;
+							why = match;
+						}
+						else {
+							keep = TRUE;
+						}
 					}
+					grpskill = grpskill->next;
+				}
+			
+				/* now do tie-breaking */
+				if(del == FALSE && keep == FALSE) {
+				/* match nothing */
+					match = REASON_NONE;
+				}
+				else if(del != keep) {
+				/* either keep or del */
+					match = ( del == TRUE) ? why : REASON_NONE;
 				}
 				else {
-					if(match != REASON_NONE) {
-						del = TRUE;
-						why = match;
-					}
-					else {
-						keep = TRUE;
-					}
-				}
-				grpskill = grpskill->next;
-			}
-			
-			/* now do tie-breaking */
-			if(del == FALSE && keep == FALSE) {
-				/* match nothing */
-				match = REASON_NONE;
-			}
-			else if(del != keep) {
-				/* either keep or del */
-				match = ( del == TRUE) ? why : REASON_NONE;
-			}
-			else {
 				/* do tie-breaking */
-				match = (tie_delete == TRUE) ? REASON_TIE : REASON_NONE;
+					match = (tie_delete == TRUE) ? REASON_TIE : REASON_NONE;
+				}
 			}
 		}
 		
@@ -237,7 +259,7 @@ int do_one_line(PMaster master, char *group, char *linein, PGrpList grpskill, in
 				}
 				if(masterk->xover_log_long == TRUE) {
 					/* print the xover formatted to look like a message header */
-					overv = masterk->xoverview;
+					overv = master->xoverview;
 					while(overv != NULL ) {
 						tmp = overv->field[overv->fieldlen]; /* null terminate it */
 						overv->field[overv->fieldlen] = '\0';
@@ -261,13 +283,11 @@ void get_xoverview(PMaster master) {
 	/* get in the xoverview.fmt list, so we can parse what xover returns later */
 	/* we'll put em in a linked list */
 	
-	int done, retval, len;
+	int done, retval, len, full;
 	char *resp;
 	POverview tmp, tmp2, curptr;
-	PKillStruct killp;
 	
 	retval = RETVAL_OK;
-	killp = master->xoverp;
 	curptr = NULL;  /* where we are currently at in the linked list */
 	
 	if(send_command(master, "list overview.fmt\r\n", &resp, 215) == RETVAL_OK) {
@@ -284,7 +304,9 @@ void get_xoverview(PMaster master) {
 			else if(retval == RETVAL_OK) {
 				/* now save em if we haven't run out of memory */
 				len = strlen(resp);
-					
+                                /* does this have a full flag on it, which means the field name */
+                                /* will be in the xover string */
+				full = (strstr(resp, ":full") == NULL) ?  FALSE : TRUE; 
 				/* now get rid of everything back to : */
 				while(resp[len] != ':') {
 					resp[len--] = '\0';
@@ -301,16 +323,20 @@ void get_xoverview(PMaster master) {
 				}
 				else {
 					/* initialize the structure */
+
+					/* do this first so we don't wipe out with null termination */ 
 					strncpy(tmp->header, resp, len); /* so we get the colon */
 					tmp->header[len] = '\0';
 					tmp->next = NULL;
 					tmp->field = NULL;
 					tmp->fieldlen = 0;
+					tmp->full = full;
+		
 					
 					if(curptr == NULL) {
 						/* at head of list */
 						curptr = tmp;
-						killp->xoverview = tmp;
+						master->xoverview = tmp;
 					}
 					else {
 						/* add to linked list */
@@ -323,7 +349,7 @@ void get_xoverview(PMaster master) {
 	}
 	if(retval != RETVAL_OK) {
 		/* free up whatever alloced */
-		tmp = killp->xoverview;
+		tmp = master->xoverview;
 		while (tmp != NULL) {
 			tmp2 = tmp;
 			if(tmp->header != NULL) {
@@ -332,13 +358,13 @@ void get_xoverview(PMaster master) {
 			free(tmp);
 			tmp = tmp2;
 		}
-		killp->xoverview = NULL;
+		master->xoverview = NULL;
 	}
-	if(master->debug == TRUE && (tmp = killp->xoverview) != NULL) {
+	if(master->debug == TRUE && (tmp = master->xoverview) != NULL) {
 		do_debug("--Xoverview.fmt list\n");
 		while(tmp != NULL) {
 			if(tmp->header != NULL) {
-				do_debug("item = %s\n", tmp->header);
+				do_debug("item = %s -- full = %s\n", tmp->header, true_str(tmp->full));
 			}
 			tmp = tmp->next;
 		}
@@ -359,82 +385,72 @@ int chk_a_group(PMaster master, POneKill grp, POverview overv, char **reason) {
 	pmy_regex ptr;
 	POverview tmp;
 	*reason = reasonstr;
-	
-	if(grp->bodybig > 0 || grp->bodysmall > 0) {
-		tmp = overv;
-		while(tmp != NULL && match == REASON_NONE) {
-			if(strcasecmp(tmp->header, "Bytes:") == 0) {
-				sscanf(tmp->field, "%lu", &bytes); /* convert ascii to long */
-				if((grp->bodybig > 0) && (bytes > grp->bodybig)) {
-					match = REASON_HIBYTES;
-				}
-				else if((grp->bodysmall > 0) && (bytes < grp->bodysmall)) {
-					match = REASON_LOWBYTES;
-				}
-			}
-			tmp = tmp->next;
-		}
-	}
-	
-	if(match == REASON_NONE && (grp->hilines > 0 || grp->lowlines > 0)) {
-		tmp = overv;
-		while(tmp != NULL && match == REASON_NONE) {
-			if(strcasecmp(tmp->header, "Lines:") == 0) {
-				sscanf(tmp->field, "%d", &lines);
-				if((grp->hilines > 0) && (lines > grp->hilines)) {
-					match = REASON_HILINES;
-				}
-				else if((grp->lowlines > 0) && (lines < grp->lowlines)) {
-					match = REASON_LOWLINES;
-				}
-			}
-			tmp = tmp->next;
-		}
-	}
-	/* now check nrxref */
-	if(match == REASON_NONE && grp->maxxref > 0) {
-		tmp = overv;
-		while(tmp != NULL && match == REASON_NONE) {
-			if(strcasecmp(tmp->header, "Xref:") == 0) {
-                                /* count the nr of commas in the group line */
-				i = 1;	/* have at least one group */
-				tptr = tmp->field;
-				while(i <= grp->maxxref && *tptr != '\0' ) {
-				/* some news server use space vice comma for separator */
-					if(*tptr == COMMA || *tptr == SPACE ) {
-						i++;
-					}
-					tptr++;
-				}
-				if(i > grp->maxxref) {
-					match = REASON_XREF;
-				}		
-			}
-			tmp = tmp->next;
-		}
-	}
 
-	if(match == REASON_NONE) {
-		ptr = grp->list;
-		while (match == REASON_NONE && ptr != NULL) {
-			tmp = overv;
-			while( tmp != NULL && match == REASON_NONE) {
-				if(strcasecmp(tmp->header, ptr->header) == 0) {
-                                        /* we need to null terminate the field and restore it later */
-					tchar = tmp->field[tmp->fieldlen];
-					tmp->field[tmp->fieldlen] = '\0';
-					if(regex_block(tmp->field, ptr, master->debug) == TRUE) {
-						match = REASON_HEADER;
-						sprintf(reasonstr, "%s-%s%s", xover_reasons[REASON_HEADER], tmp->header, tmp->field);
+	tmp = overv;
+	/* go thru each header field, and see if we must test against it */
+	while ( tmp != NULL && match == REASON_NONE) {
+		/* only do the test if we have a valid header & field to test against */
+		if(tmp->field != NULL && tmp->header != NULL) {
+			/* test the size of the body of the article */
+			if(grp->bodybig > 0 || grp->bodysmall > 0) {
+				if(strcasecmp(tmp->header, "Bytes:") == 0) {
+					sscanf(tmp->field, "%lu", &bytes); /* convert ascii to long */
+					if((grp->bodybig > 0) && (bytes > grp->bodybig)) {
+						match = REASON_HIBYTES;
 					}
-					tmp->field[tmp->fieldlen] = tchar;
+					else if((grp->bodysmall > 0) && (bytes < grp->bodysmall)) {
+						match = REASON_LOWBYTES;
+					}
+				}	
+			}
+			/* test the number of lines in the article */
+			if((match == REASON_NONE) && (grp->hilines > 0 || grp->lowlines > 0)) {
+				if(strcasecmp(tmp->header, "Lines:") == 0) {
+					sscanf(tmp->field, "%d", &lines);
+					if((grp->hilines > 0) && (lines > grp->hilines)) {
+						match = REASON_HILINES;
+					}
+					else if((grp->lowlines > 0) && (lines < grp->lowlines)) {
+						match = REASON_LOWLINES;
+					}
 				}
-				tmp = tmp->next;
+			}
+			/* check the number of xrefs */
+			if(match == REASON_NONE && grp->maxxref > 0) {
+				if(strcasecmp(tmp->header, "Xref:") == 0) {
+					i = 0;
+					tptr = tmp->field;
+					while(i <= grp->maxxref && *tptr != '\0' ) {
+						if(*tptr == COLON ) {
+							i++;
+						}
+						tptr++;
+					}
+					if(i > grp->maxxref) {
+						match = REASON_XREF;
+					}
+				}
+			}
+			/* match against any of the headers */
+			if(match == REASON_NONE) {
+				ptr = grp->list;
+				while(match == REASON_NONE && ptr != NULL) {
+					if(strcasecmp(tmp->header, ptr->header) == 0) {
+						/* we need to null terminate the field and restore it later */
+						tchar = tmp->field[tmp->fieldlen];
+						tmp->field[tmp->fieldlen] = '\0';
+						if(regex_block(tmp->field, ptr, master->debug) == TRUE) {
+							match = REASON_HEADER;
+							sprintf(reasonstr, "%s-%s%s", xover_reasons[REASON_HEADER], tmp->header, tmp->field);
+						}
+						tmp->field[tmp->fieldlen] = tchar;
+					}
+					ptr = ptr->next;
+				}
 			}
-			ptr = ptr->next;
 		}
+		tmp = tmp->next;
 	}
-	
 	return match;
 }
 /*---------------------------------------------------------------------------*/
@@ -461,6 +477,113 @@ int match_group(char *match_grp, char *group, int debug) {
 	}
 	return match;
 }
+/*-------------------------------------------------------------------------------*/
+int get_xover(PMaster master, char *group, long startnr, long endnr) {
+	
+	int len, nr, done, retval = RETVAL_OK;
+	char cmd[MAXLINLEN], *resp, *ptr, *msgid;
+	long msgnr;
+	
+	sprintf(cmd, "xover %ld-%ld\r\n", startnr,endnr);
+	retval = send_command(master, cmd, &resp, 0);
+	if(retval == RETVAL_OK) {
+		number(resp, &nr);
+		switch(nr) {
+		  default:
+		  case 412:
+		  case 502:
+			  /* abort on these errors */
+			  error_log(ERRLOG_REPORT, xover_phrases[0], resp, NULL);
+			  retval = RETVAL_NOXOVER;
+			  break;
+		  case 420:
+			  /* no articles available, no prob */
+			  break;
+		  case 224:
+			  /* got a list coming at us */
+			  done = FALSE;
+			  while((done == FALSE) && (retval == RETVAL_OK)) {
+				  len = sgetline(master->sockfd, &resp);
+				  if(master->debug == TRUE) {
+					  do_debug("Got xover line: %s", resp);
+				  }
+				  
+				  /* have we got the last one? */
+				  if((len == 2) && (strcmp(resp, ".\n") == 0)) {
+					  done = TRUE;
+				  }
+				  else if(len < 0) {
+					  retval =RETVAL_ERROR;
+				  }
+				  else {
+					  ptr  = get_long(resp, &msgnr);  /* get our message number */
+					  msgid = find_msgid(master, ptr);        /* find the msg-id */
+					  if(msgid == NULL) {
+						  error_log(ERRLOG_REPORT, xover_phrases[13], resp);
+					  }
+					  else {						  
+						  /* alloc the memory for it */
+						  retval= allocnode(master, msgid, MANDATORY_OPTIONAL, group, msgnr);
+					  }
+					  
+				  }
+			  }
+			  break;
+		}
+	}
+	return retval;
+}
+/*-----------------------------------------------------------------------------------*/
+/* find the msgid in a xover, and return it null-terminated, so can alloc memory, etc*/
+/*-----------------------------------------------------------------------------------*/
+char *find_msgid(PMaster master, char *xover) {
 
+	char *ptr, *ptr2, *retval = NULL;
+	POverview pov;
+
+	pov = master->xoverview;
+	ptr = xover;
+
+	/* now go thru and find the Message-ID in the xoverview, using the overview.fmt */
+	/* to tell us which field is which */
+
+	if(ptr != NULL) {
+		while ( *ptr != '\0' && pov != NULL && strcmp(pov->header, "Message-ID:") != 0) {
+			/* go to the next field, past the tab */
+			pov = pov->next;
+			while(*ptr != '\t' && *ptr != '\0') {
+				ptr++;
+			}
+			if(*ptr != '\0') {
+				ptr++; /* past the tab */
+			}
+		}
+		if(*ptr != '\0' && pov != NULL) {
+			/* bingo, found it */
+			/* find the start of the msgid */
+			while(*ptr != '\0' && *ptr != '<') {
+				ptr++;
+			}
+			if(ptr != '\0') {
+				ptr2 = ptr;
+				/* NULL terminate the msgid */
+				while(*ptr2 != '\0' && *ptr2 != '>') {
+					ptr2++;
+				}
+				/* ONLY if we find a valid start and end to the MessageId */
+				/* do we set a valid return value */
+				if(*ptr2 != '\0') {
+					*(ptr2+1) = '\0';
+					retval = ptr;
+				}
+			}
+		}
+	}
+
+	return retval;
+}
+
+	
+	
 
 	
